@@ -1,3 +1,12 @@
+#  File R/ergmMPLE.R in package ergm, part of the Statnet suite
+#  of packages for network analysis, http://statnet.org .
+#
+#  This software is distributed under the GPL-3 license.  It is free,
+#  open source, and has the attribution requirements (GPL Section 7) at
+#  http://statnet.org/attribution
+#
+#  Copyright 2003-2017 Statnet Commons
+#######################################################################
 ############################################################################
 # The <ergmMPLE> function has different behavior based on whether the given
 # formula should be fit or not. If so, <ergm> is called. If not, the elements
@@ -43,6 +52,7 @@ ergmMPLE <- function(formula, fitmodel=FALSE, output=c("matrix", "array", "fit")
       if(fitmodel) output <- "fit"
   }
   check.control.class("ergm")
+  control.toplevel(...,myname="ergm")
   output <- match.arg(output)
   if (output=="fit") {
     return(ergm(formula, estimate="MPLE", control=control, verbose=verbose, ...))
