@@ -1,3 +1,12 @@
+#  File R/logLik.ergm.R in package ergm, part of the Statnet suite
+#  of packages for network analysis, http://statnet.org .
+#
+#  This software is distributed under the GPL-3 license.  It is free,
+#  open source, and has the attribution requirements (GPL Section 7) at
+#  http://statnet.org/attribution
+#
+#  Copyright 2003-2017 Statnet Commons
+#######################################################################
 ## A function to compute and return the log-likelihood of an ERGM MLE.
 logLik.ergm<-function(object, add=FALSE, force.reeval=FALSE, eval.loglik=add || force.reeval, control=control.logLik.ergm(), ...){
 
@@ -6,7 +15,8 @@ logLik.ergm<-function(object, add=FALSE, force.reeval=FALSE, eval.loglik=add ||
   # Then, we need to recalculate...
   
   check.control.class()
-  
+  control.toplevel(...)
+ 
   control.transfer <- c("MCMC.burnin", "MCMC.interval", "MCMC.prop.weights",
 "MCMC.prop.args", "MCMC.packagenames", "MCMC.init.maxedges", "MCMC.samplesize",
 "obs.MCMC.burnin", "obs.MCMC.interval", "obs.MCMC.samplesize","warn.dyads","MPLE.type","MPLE.max.dyad.types","parallel","parallel.type","parallel.version.check"
@@ -85,6 +95,10 @@ nologLik.message<-function(objname){
 logLikNull <- function(object, ...) UseMethod("logLikNull")
 
 logLikNull.ergm <- function(object, control=control.logLik.ergm(), ...){
+  check.control.class("logLik.ergm")
+
+  control.toplevel(..., myname="logLik.ergm")
+  
   if(!is.null(object$null.lik)) object$null.lik
 
   nobs <- if(is.null(object$mle.lik)) network.dyadcount(object$network,FALSE) - network.edgecount(NVL(get.miss.dyads(object$constrained, object$constrained.obs),network.initialize(1))) else attr(object$mle.lik,"nobs")
