@@ -5,7 +5,7 @@
  *  open source, and has the attribution requirements (GPL Section 7) at
  *  http://statnet.org/attribution
  *
- *  Copyright 2003-2013 Statnet Commons
+ *  Copyright 2003-2017 Statnet Commons
  */
 #ifndef WTCHANGESTAT_H
 #define WTCHANGESTAT_H
@@ -120,7 +120,7 @@ typedef struct WtModelTermstruct {
 #define TOGGLEIND toggleind_var
 
 /* macro to set all changestats to zero at start of function */
-#define ZERO_ALL_CHANGESTATS() for(unsigned int TOGGLEIND=0; TOGGLEIND<N_CHANGE_STATS; TOGGLEIND++) CHANGE_STAT[TOGGLEIND]=0.0
+#define ZERO_ALL_CHANGESTATS() memset(CHANGE_STAT, 0, sizeof(double)*N_CHANGE_STATS);
 
 /* Cycle through all toggles proposed for the current step, then
    make the current toggle in case of more than one proposed toggle, then
