@@ -1,3 +1,12 @@
+/*  File src/CD.c in package ergm, part of the Statnet suite
+ *  of packages for network analysis, http://statnet.org .
+ *
+ *  This software is distributed under the GPL-3 license.  It is free,
+ *  open source, and has the attribution requirements (GPL Section 7) at
+ *  http://statnet.org/attribution
+ *
+ *  Copyright 2003-2017 Statnet Commons
+ */
 #include "CD.h"
 
 /*****************
@@ -179,8 +188,11 @@ MCMCStatus CDStep(MHproposal *MHp,
 	    Rprintf("Too many MH Proposal function failures.\n");
 	    return MCMC_MH_FAILED;
 	  }
-	case MH_CONSTRAINT:
 	  continue;
+	  
+	case MH_CONSTRAINT:
+	  MHp->logratio = -INFINITY; // Force rejection of proposal.
+	  break; // Do not attempt any more proposals in this multiplicity chain.
 	}
       }
       
