@@ -1,3 +1,12 @@
+/*  File src/changestats.dgw_sp.c in package ergm, part of the Statnet suite
+ *  of packages for network analysis, http://statnet.org .
+ *
+ *  This software is distributed under the GPL-3 license.  It is free,
+ *  open source, and has the attribution requirements (GPL Section 7) at
+ *  http://statnet.org/attribution
+ *
+ *  Copyright 2003-2017 Statnet Commons
+ */
 #include "changestats.dgw_sp.h"
 
 
@@ -32,25 +41,22 @@ void dspUTP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
 
   FOR_EACH_TOGGLE(i) {
     L2th=0;
-    echange = (EdgetreeSearch(tail=TAIL(i), head=HEAD(i), nwp->outedges) == 0) ? 1 : -1;
+    echange = (IS_OUTEDGE(tail=TAIL(i), head=HEAD(i)) == 0) ? 1 : -1;
     /* step through outedges of head */
-    for(e = EdgetreeMinimum(nwp->outedges, head); 
-    (u = nwp->outedges[e].value) != 0; e = EdgetreeSuccessor(nwp->outedges, e)) {
-      if (EdgetreeSearch(MIN(u,tail), MAX(u,tail), nwp->outedges) != 0){
+    STEP_THROUGH_OUTEDGES(head,e,u){
+      if (IS_UNDIRECTED_EDGE(u,tail) != 0){
         L2th++;
         L2tu=0;
         L2uh=0;
         /* step through outedges of u */
-        for(f = EdgetreeMinimum(nwp->outedges, u); (v = nwp->outedges[f].value) != 0;
-        f = EdgetreeSuccessor(nwp->outedges, f)){
-          if(EdgetreeSearch(MIN(v,head),MAX(v,head),nwp->outedges)!= 0) L2uh++;
-          if(EdgetreeSearch(MIN(v,tail),MAX(v,tail),nwp->outedges)!= 0) L2tu++;
+        STEP_THROUGH_OUTEDGES(u,f,v){
+          if(IS_UNDIRECTED_EDGE(v,head)!= 0) L2uh++;
+          if(IS_UNDIRECTED_EDGE(v,tail)!= 0) L2tu++;
         }
         /* step through inedges of u */
-        for(f = EdgetreeMinimum(nwp->inedges, u); (v = nwp->inedges[f].value) != 0;
-        f = EdgetreeSuccessor(nwp->inedges, f)){
-          if(EdgetreeSearch(MIN(v,head),MAX(v,head),nwp->outedges)!= 0) L2uh++;
-          if(EdgetreeSearch(MIN(v,tail),MAX(v,tail),nwp->outedges)!= 0) L2tu++;
+        STEP_THROUGH_INEDGES(u,f,v){
+          if(IS_UNDIRECTED_EDGE(v,head)!= 0) L2uh++;
+          if(IS_UNDIRECTED_EDGE(v,tail)!= 0) L2tu++;
         }
         for(j = 0; j < nd; j++){
           deg = (Vertex)dvec[j];
@@ -60,23 +66,20 @@ void dspUTP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
       }
     }
     /* step through inedges of head */
-    for (e = EdgetreeMinimum(nwp->inedges, head); (u = nwp->inedges[e].value) != 0;
-    e = EdgetreeSuccessor(nwp->inedges, e)){
-      if (EdgetreeSearch(MIN(u,tail), MAX(u,tail), nwp->outedges) != 0){
+    STEP_THROUGH_INEDGES(head,e,u){
+      if (IS_UNDIRECTED_EDGE(u,tail) != 0){
         L2th++;
         L2tu=0;
         L2uh=0;
         /* step through outedges of u */
-        for(f = EdgetreeMinimum(nwp->outedges, u); (v = nwp->outedges[f].value) != 0;
-        f = EdgetreeSuccessor(nwp->outedges, f)){
-          if(EdgetreeSearch(MIN(v,head),MAX(v,head),nwp->outedges)!= 0) L2uh++;
-          if(EdgetreeSearch(MIN(v,tail),MAX(v,tail),nwp->outedges)!= 0) L2tu++;
+        STEP_THROUGH_OUTEDGES(u,f,v){
+          if(IS_UNDIRECTED_EDGE(v,head)!= 0) L2uh++;
+          if(IS_UNDIRECTED_EDGE(v,tail)!= 0) L2tu++;
         }
         /* step through inedges of u */
-        for(f = EdgetreeMinimum(nwp->inedges, u); (v = nwp->inedges[f].value) != 0;
-        f = EdgetreeSuccessor(nwp->inedges, f)){
-          if(EdgetreeSearch(MIN(v,head),MAX(v,head),nwp->outedges)!= 0) L2uh++;
-          if(EdgetreeSearch(MIN(v,tail),MAX(v,tail),nwp->outedges)!= 0) L2tu++;
+        STEP_THROUGH_INEDGES(u,f,v){
+          if(IS_UNDIRECTED_EDGE(v,head)!= 0) L2uh++;
+          if(IS_UNDIRECTED_EDGE(v,tail)!= 0) L2tu++;
         }
         for(j = 0; j < nd; j++){
           deg = (Vertex)dvec[j];
@@ -346,19 +349,17 @@ void dspRTP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
 
   FOR_EACH_TOGGLE(i) {
     L2th=0;
-    echange = (EdgetreeSearch(tail=TAIL(i), head=HEAD(i), nwp->outedges) == 0) ? 1 : -1;
+    echange = (IS_OUTEDGE(tail=TAIL(i), head=HEAD(i)) == 0) ? 1 : -1;
     htedge=IS_OUTEDGE(head,tail);  /*Is there an h->t (reciprocating) edge?*/
     /* step through inedges of tail (k->t: k!=h,h->t,k<->h)*/
-    for(e = EdgetreeMinimum(nwp->inedges, tail); 
-    (k = nwp->inedges[e].value) != 0; e = EdgetreeSuccessor(nwp->inedges, e)) {
+    STEP_THROUGH_INEDGES(tail,e,k){
       if(k!=head){
         /*Do we have a t<->k<->h TP?  If so, add it to our count.*/
         L2th+=(IS_OUTEDGE(tail,k)&&IS_OUTEDGE(head,k)&&IS_OUTEDGE(k,head));
         if(htedge&&IS_OUTEDGE(head,k)&&IS_OUTEDGE(k,head)){ /*Only consider stats that could change*/
           L2kt=0;
           /*Now, count # u such that k<->u<->t (to get (k,t)'s ESP value)*/
-          for(f = EdgetreeMinimum(nwp->outedges, k); 
-          (u = nwp->outedges[f].value) != 0; f = EdgetreeSuccessor(nwp->outedges, f)) { 
+          STEP_THROUGH_OUTEDGES(k,f,u){ 
             if((u!=tail)&&(IS_OUTEDGE(u,k)))
               L2kt+=(IS_OUTEDGE(u,tail)&&IS_OUTEDGE(tail,u));  /*k<->u<->t?*/
           }
@@ -371,14 +372,12 @@ void dspRTP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
       }
     }
     /* step through outedges of tail (t->k: k!=h,h->t,k<->h)*/
-    for(e = EdgetreeMinimum(nwp->outedges, tail); 
-    (k = nwp->outedges[e].value) != 0; e = EdgetreeSuccessor(nwp->outedges, e)) {
+    STEP_THROUGH_OUTEDGES(tail,e,k) {
       if(k!=head){
         if(htedge&&IS_OUTEDGE(head,k)&&IS_OUTEDGE(k,head)){ /*Only consider stats that could change*/
           L2tk=0;
           /*Now, count # u such that k<->u<->t (to get (tk)'s ESP value)*/
-          for(f = EdgetreeMinimum(nwp->outedges, k); 
-          (u = nwp->outedges[f].value) != 0; f = EdgetreeSuccessor(nwp->outedges, f)) { 
+          STEP_THROUGH_OUTEDGES(k,f,u){ 
             if((u!=tail)&&(IS_OUTEDGE(u,k)))
               L2tk+=(IS_OUTEDGE(u,tail)&&IS_OUTEDGE(tail,u));  /*k<->u<->t?*/
           }
@@ -391,14 +390,12 @@ void dspRTP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
       }
     }
     /* step through inedges of head (k->h: k!=t,h->t,k<->t)*/
-    for(e = EdgetreeMinimum(nwp->inedges, head); 
-    (k = nwp->inedges[e].value) != 0; e = EdgetreeSuccessor(nwp->inedges, e)) {
+    STEP_THROUGH_INEDGES(head,e,k){
       if(k!=tail){
         if(htedge&&IS_OUTEDGE(tail,k)&&IS_OUTEDGE(k,tail)){ /*Only consider stats that could change*/
           L2kh=0;
           /*Now, count # u such that k<->u<->h (to get k->h's ESP value)*/
-          for(f = EdgetreeMinimum(nwp->outedges, k); 
-          (u = nwp->outedges[f].value) != 0; f = EdgetreeSuccessor(nwp->outedges, f)) { 
+          STEP_THROUGH_OUTEDGES(k,f,u){ 
             if((u!=head)&&(IS_OUTEDGE(u,k)))
               L2kh+=(IS_OUTEDGE(u,head)&&IS_OUTEDGE(head,u));  /*k<->u<->h?*/
           }
@@ -411,14 +408,12 @@ void dspRTP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
       }
     }
     /* step through outedges of head (h->k: k!=t,h->t,k<->t)*/
-    for(e = EdgetreeMinimum(nwp->outedges, head); 
-    (k = nwp->outedges[e].value) != 0; e = EdgetreeSuccessor(nwp->outedges, e)) {
+    STEP_THROUGH_OUTEDGES(head,e,k){
       if(k!=tail){
         if(htedge&&IS_OUTEDGE(tail,k)&&IS_OUTEDGE(k,tail)){ /*Only consider stats that could change*/
           L2hk=0;
           /*Now, count # u such that k<->u<->h (to get h->k's ESP value)*/
-          for(f = EdgetreeMinimum(nwp->outedges, k); 
-          (u = nwp->outedges[f].value) != 0; f = EdgetreeSuccessor(nwp->outedges, f)) { 
+          STEP_THROUGH_OUTEDGES(k,f,u){ 
             if((u!=head)&&(IS_OUTEDGE(u,k)))
               L2hk+=(IS_OUTEDGE(u,head)&&IS_OUTEDGE(head,u));  /*k<->u<->h?*/
           }
@@ -461,7 +456,7 @@ Type codes are as follows (where (i,j) is the focal edge):
 Only one type may be specified per esp term.  UTP should always be used for undirected graphs; OTP is the traditional directed default.
 */
 D_CHANGESTAT_FN(d_ddsp) { 
-  int i,type;
+  int type;
   double *dvec,*cs;
   
   /*Set things up*/
@@ -509,8 +504,8 @@ D_CHANGESTAT_FN(d_dgwdsp) {
   oneexpa = 1.0-exp(-alpha);    /*Precompute (1-exp(-alpha))*/
   type=(int)INPUT_PARAM[1];     /*Get the ESP type code to be used*/
   maxesp=(int)INPUT_PARAM[2];   /*Get the max ESP cutoff to use*/
-  cs=Calloc(maxesp,double);     /*Allocate memory for the ESP changescores*/
-  dvec=Calloc(maxesp,double);   /*Allocate memory for the ESP vals*/
+  cs=alloca(maxesp*sizeof(double));     /*Allocate memory for the ESP changescores*/
+  dvec=alloca(maxesp*sizeof(double));   /*Allocate memory for the ESP vals*/
   for(i=0;i<maxesp;i++)         /*Initialize the ESP vals*/
     dvec[i]=i+1.0;
 
@@ -532,10 +527,6 @@ D_CHANGESTAT_FN(d_dgwdsp) {
     }
   }
   CHANGE_STAT[0]*=exp(alpha);
-  
-  /*Free the changescore memory*/
-  Free(cs);
-  Free(dvec);
 }
 
 
@@ -571,25 +562,22 @@ void espUTP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
 
   FOR_EACH_TOGGLE(i) {
     L2th=0;
-    echange = (EdgetreeSearch(tail=TAIL(i), head=HEAD(i), nwp->outedges) == 0) ? 1 : -1;
+    echange = (IS_OUTEDGE(tail=TAIL(i), head=HEAD(i)) == 0) ? 1 : -1;
     /* step through outedges of head */
-    for(e = EdgetreeMinimum(nwp->outedges, head); 
-    (u = nwp->outedges[e].value) != 0; e = EdgetreeSuccessor(nwp->outedges, e)) {
-      if (EdgetreeSearch(MIN(u,tail), MAX(u,tail), nwp->outedges) != 0){
+    STEP_THROUGH_OUTEDGES(head,e,u){
+      if (IS_UNDIRECTED_EDGE(u,tail) != 0){
         L2th++;
         L2tu=0;
         L2uh=0;
         /* step through outedges of u */
-        for(f = EdgetreeMinimum(nwp->outedges, u); (v = nwp->outedges[f].value) != 0;
-        f = EdgetreeSuccessor(nwp->outedges, f)){
-          if(EdgetreeSearch(MIN(v,head),MAX(v,head),nwp->outedges)!= 0) L2uh++;
-          if(EdgetreeSearch(MIN(v,tail),MAX(v,tail),nwp->outedges)!= 0) L2tu++;
+        STEP_THROUGH_OUTEDGES(u,f,v){
+          if(IS_UNDIRECTED_EDGE(v,head)!= 0) L2uh++;
+          if(IS_UNDIRECTED_EDGE(v,tail)!= 0) L2tu++;
         }
         /* step through inedges of u */
-        for(f = EdgetreeMinimum(nwp->inedges, u); (v = nwp->inedges[f].value) != 0;
-        f = EdgetreeSuccessor(nwp->inedges, f)){
-          if(EdgetreeSearch(MIN(v,head),MAX(v,head),nwp->outedges)!= 0) L2uh++;
-          if(EdgetreeSearch(MIN(v,tail),MAX(v,tail),nwp->outedges)!= 0) L2tu++;
+        STEP_THROUGH_INEDGES(u,f,v){
+          if(IS_UNDIRECTED_EDGE(v,head)!= 0) L2uh++;
+          if(IS_UNDIRECTED_EDGE(v,tail)!= 0) L2tu++;
         }
         for(j = 0; j < nd; j++){
           deg = (Vertex)dvec[j];
@@ -599,23 +587,20 @@ void espUTP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
       }
     }
     /* step through inedges of head */
-    for (e = EdgetreeMinimum(nwp->inedges, head); (u = nwp->inedges[e].value) != 0;
-    e = EdgetreeSuccessor(nwp->inedges, e)){
-      if (EdgetreeSearch(MIN(u,tail), MAX(u,tail), nwp->outedges) != 0){
+    STEP_THROUGH_INEDGES(head,e,u){
+      if (IS_UNDIRECTED_EDGE(u,tail) != 0){
         L2th++;
         L2tu=0;
         L2uh=0;
         /* step through outedges of u */
-        for(f = EdgetreeMinimum(nwp->outedges, u); (v = nwp->outedges[f].value) != 0;
-        f = EdgetreeSuccessor(nwp->outedges, f)){
-          if(EdgetreeSearch(MIN(v,head),MAX(v,head),nwp->outedges)!= 0) L2uh++;
-          if(EdgetreeSearch(MIN(v,tail),MAX(v,tail),nwp->outedges)!= 0) L2tu++;
+        STEP_THROUGH_OUTEDGES(u,f,v){
+          if(IS_UNDIRECTED_EDGE(v,head)!= 0) L2uh++;
+          if(IS_UNDIRECTED_EDGE(v,tail)!= 0) L2tu++;
         }
         /* step through inedges of u */
-        for(f = EdgetreeMinimum(nwp->inedges, u); (v = nwp->inedges[f].value) != 0;
-        f = EdgetreeSuccessor(nwp->inedges, f)){
-          if(EdgetreeSearch(MIN(v,head),MAX(v,head),nwp->outedges)!= 0) L2uh++;
-          if(EdgetreeSearch(MIN(v,tail),MAX(v,tail),nwp->outedges)!= 0) L2tu++;
+        STEP_THROUGH_INEDGES(u,f,v){
+          if(IS_UNDIRECTED_EDGE(v,head)!= 0) L2uh++;
+          if(IS_UNDIRECTED_EDGE(v,tail)!= 0) L2tu++;
         }
         for(j = 0; j < nd; j++){
           deg = (Vertex)dvec[j];
@@ -664,12 +649,11 @@ void espOTP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
   FOR_EACH_TOGGLE(i) {
     //Rprintf("Working on toggle %d (%d,%d)\n",i,TAIL(i),HEAD(i));
     L2th=0;
-    echange = (EdgetreeSearch(tail=TAIL(i), head=HEAD(i), nwp->outedges) == 0) ? 1 : -1;
+    echange = (IS_OUTEDGE(tail=TAIL(i), head=HEAD(i)) == 0) ? 1 : -1;
     //Rprintf("\tEdge change is %d\n",echange);
     /* step through outedges of tail (i.e., k: t->k)*/
     //Rprintf("Walking through outedges of tail\n",echange);
-    for(e = EdgetreeMinimum(nwp->outedges, tail); 
-    (k = nwp->outedges[e].value) != 0; e = EdgetreeSuccessor(nwp->outedges, e)) {
+    STEP_THROUGH_OUTEDGES(tail,e,k) {
       if((k!=head)&&(IS_OUTEDGE(k,head))){
         /*We have a t->k->h two-path, so add it to our count.*/
         L2th++;
@@ -678,8 +662,7 @@ void espOTP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
         //Rprintf("\tk==%d, passed criteria\n",k);
         L2tk=0;
         /*Now, count # u such that t->u->k (to find t->k's ESP value)*/
-        for(f = EdgetreeMinimum(nwp->inedges, k); 
-        (u = nwp->inedges[f].value) != 0; f = EdgetreeSuccessor(nwp->inedges, f)) {
+        STEP_THROUGH_INEDGES(k,f,u){
           //Rprintf("\t\tTrying u==%d\n",u);
           if(u!=tail) 
             L2tk+=IS_OUTEDGE(tail,u); /*Increment if there is a trans edge*/
@@ -694,14 +677,12 @@ void espOTP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
     }
     /* step through inedges of head (i.e., k: k->h)*/
     //Rprintf("Walking through inedges of head\n",k);
-    for(e = EdgetreeMinimum(nwp->inedges, head); 
-    (k = nwp->inedges[e].value) != 0; e = EdgetreeSuccessor(nwp->inedges, e)) {
+    STEP_THROUGH_INEDGES(head,e,k){
       if((k!=tail)&&(IS_OUTEDGE(k,tail))){ /*Only use contingent cases*/
         //Rprintf("\tk==%d, passed criteria\n",k);
         L2kh=0;
         /*Now, count # u such that k->u->j (to find k->h's ESP value)*/
-        for(f = EdgetreeMinimum(nwp->outedges, k); 
-        (u = nwp->outedges[f].value) != 0; f = EdgetreeSuccessor(nwp->outedges, f)) {
+        STEP_THROUGH_OUTEDGES(k,f,u){
           if(u!=head) 
             L2kh+=IS_OUTEDGE(u,head); /*Increment if there is a trans edge*/
         }
@@ -749,20 +730,18 @@ void espITP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
   FOR_EACH_TOGGLE(i) {
     //Rprintf("Working on toggle %d (%d,%d)\n",i,TAIL(i),HEAD(i));
     L2th=0;
-    echange = (EdgetreeSearch(tail=TAIL(i), head=HEAD(i), nwp->outedges) == 0) ? 1 : -1;
+    echange = (IS_OUTEDGE(tail=TAIL(i), head=HEAD(i)) == 0) ? 1 : -1;
     //Rprintf("\tEdge change is %d\n",echange);
     /* step through outedges of head (i.e., k: h->k)*/
     //Rprintf("Walking through outedges of head\n",echange);
-    for(e = EdgetreeMinimum(nwp->outedges, head); 
-    (k = nwp->outedges[e].value) != 0; e = EdgetreeSuccessor(nwp->outedges, e)) {
+    STEP_THROUGH_OUTEDGES(head,e,k){
       if((k!=tail)&&(IS_OUTEDGE(k,tail))){ /*Only use contingent cases*/
         //Rprintf("\tk==%d, passed criteria\n",k);
         /*We have a h->k->t two-path, so add it to our count.*/
         L2th++;
         L2hk=0;
         /*Now, count # u such that k->u->h (so that we know k's ESP value)*/
-        for(f = EdgetreeMinimum(nwp->outedges, k); 
-        (u = nwp->outedges[f].value) != 0; f = EdgetreeSuccessor(nwp->outedges, f)) {
+        STEP_THROUGH_OUTEDGES(k,f,u){
           //Rprintf("\t\tTrying u==%d\n",u);
           if(u!=head) 
             L2hk+=IS_OUTEDGE(u,head); /*Increment if there is a cyclic edge*/
@@ -777,14 +756,12 @@ void espITP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
     }
     /* step through inedges of tail (i.e., k: k->t)*/
     //Rprintf("Walking through inedges of tail\n",k);
-    for(e = EdgetreeMinimum(nwp->inedges, tail); 
-    (k = nwp->inedges[e].value) != 0; e = EdgetreeSuccessor(nwp->inedges, e)) {
+    STEP_THROUGH_INEDGES(tail,e,k){
       if((k!=head)&&(IS_OUTEDGE(head,k))){ /*Only use contingent cases*/
         //Rprintf("\tk==%d, passed criteria\n",k);
         L2kt=0;
         /*Now, count # u such that t->u->k (so that we know k's ESP value)*/
-        for(f = EdgetreeMinimum(nwp->inedges, k); 
-        (u = nwp->inedges[f].value) != 0; f = EdgetreeSuccessor(nwp->inedges, f)) {
+        STEP_THROUGH_INEDGES(k,f,u){
           if(u!=tail) 
             L2kt+=IS_OUTEDGE(tail,u); /*Increment if there is a cyclic edge*/
         }
@@ -830,18 +807,16 @@ void espOSP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
 
   FOR_EACH_TOGGLE(i) {
     L2th=0;
-    echange = (EdgetreeSearch(tail=TAIL(i), head=HEAD(i), nwp->outedges) == 0) ? 1 : -1;
+    echange = (IS_OUTEDGE(tail=TAIL(i), head=HEAD(i)) == 0) ? 1 : -1;
     /* step through outedges of tail (i.e., k: t->k, k->h, k!=h)*/
-    for(e = EdgetreeMinimum(nwp->outedges, tail); 
-    (k = nwp->outedges[e].value) != 0; e = EdgetreeSuccessor(nwp->outedges, e)) {
+    STEP_THROUGH_OUTEDGES(tail,e,k) {
       if(k!=head){
         /*Do we have a t->k,h->k SP?  If so, add it to our count.*/
         L2th+=IS_OUTEDGE(head,k);
         if(IS_OUTEDGE(k,head)){ /*Only consider stats that could change*/
           L2tk=0;
           /*Now, count # u such that t->u,k->u (to get t->k's ESP value)*/
-          for(f = EdgetreeMinimum(nwp->outedges, k); 
-          (u = nwp->outedges[f].value) != 0; f = EdgetreeSuccessor(nwp->outedges, f)) { 
+          STEP_THROUGH_OUTEDGES(k,f,u){ 
             if(u!=tail)
               L2tk+=IS_OUTEDGE(tail,u);  /*Increment if there is an OSP*/
           }
@@ -854,13 +829,11 @@ void espOSP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
       }
     }
     /* step through inedges of tail (i.e., k: k->t, k->h, k!=h)*/
-    for(e = EdgetreeMinimum(nwp->inedges, tail); 
-    (k = nwp->inedges[e].value) != 0; e = EdgetreeSuccessor(nwp->inedges, e)) {
+    STEP_THROUGH_INEDGES(tail,e,k){
       if((k!=head)&&(IS_OUTEDGE(k,head))){ /*Only stats that could change*/
         L2kt=0;
         /*Now, count # u such that t->u,k->u (to get k->t's ESP value)*/
-        for(f = EdgetreeMinimum(nwp->outedges, k); 
-        (u = nwp->outedges[f].value) != 0; f = EdgetreeSuccessor(nwp->outedges, f)) { 
+        STEP_THROUGH_OUTEDGES(k,f,u){ 
           if(u!=tail)
             L2kt+=IS_OUTEDGE(tail,u);  /*Increment if there is an OSP*/
         }
@@ -904,18 +877,16 @@ void espISP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
 
   FOR_EACH_TOGGLE(i) {
     L2th=0;
-    echange = (EdgetreeSearch(tail=TAIL(i), head=HEAD(i), nwp->outedges) == 0) ? 1 : -1;
+    echange = (IS_OUTEDGE(tail=TAIL(i), head=HEAD(i)) == 0) ? 1 : -1;
     /* step through inedges of head (i.e., k: k->h, t->k, k!=t)*/
-    for(e = EdgetreeMinimum(nwp->inedges, head); 
-    (k = nwp->inedges[e].value) != 0; e = EdgetreeSuccessor(nwp->inedges, e)) {
+    STEP_THROUGH_INEDGES(head,e,k){
       if(k!=tail){
         /*Do we have a k->t,k->h SP?  If so, add it to our count.*/
         L2th+=IS_OUTEDGE(k,tail);
         if(IS_OUTEDGE(tail,k)){ /*Only consider stats that could change*/
           L2kh=0;
           /*Now, count # u such that u->h,u->k (to get h>k's ESP value)*/
-          for(f = EdgetreeMinimum(nwp->inedges, k); 
-          (u = nwp->inedges[f].value) != 0; f = EdgetreeSuccessor(nwp->inedges, f)) { 
+          STEP_THROUGH_INEDGES(k,f,u){ 
             if(u!=head)
               L2kh+=IS_OUTEDGE(u,head);  /*Increment if there is an ISP*/
           }
@@ -928,13 +899,11 @@ void espISP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
       }
     }
     /* step through outedges of head (i.e., k: h->k, t->k, k!=t)*/
-    for(e = EdgetreeMinimum(nwp->outedges, head); 
-    (k = nwp->outedges[e].value) != 0; e = EdgetreeSuccessor(nwp->outedges, e)) {
+    STEP_THROUGH_OUTEDGES(head,e,k){
       if((k!=tail)&&(IS_OUTEDGE(tail,k))){ /*Only stats that could change*/
         L2hk=0;
         /*Now, count # u such that u->h,u->k (to get k->h's ESP value)*/
-        for(f = EdgetreeMinimum(nwp->inedges, k); 
-        (u = nwp->inedges[f].value) != 0; f = EdgetreeSuccessor(nwp->inedges, f)) { 
+        STEP_THROUGH_INEDGES(k,f,u){ 
           if(u!=head)
             L2hk+=IS_OUTEDGE(u,head);  /*Increment if there is an ISP*/
         }
@@ -980,19 +949,17 @@ void espRTP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
 
   FOR_EACH_TOGGLE(i) {
     L2th=0;
-    echange = (EdgetreeSearch(tail=TAIL(i), head=HEAD(i), nwp->outedges) == 0) ? 1 : -1;
+    echange = (IS_OUTEDGE(tail=TAIL(i), head=HEAD(i)) == 0) ? 1 : -1;
     htedge=IS_OUTEDGE(head,tail);  /*Is there an h->t (reciprocating) edge?*/
     /* step through inedges of tail (k->t: k!=h,h->t,k<->h)*/
-    for(e = EdgetreeMinimum(nwp->inedges, tail); 
-    (k = nwp->inedges[e].value) != 0; e = EdgetreeSuccessor(nwp->inedges, e)) {
+    STEP_THROUGH_INEDGES(tail,e,k){
       if(k!=head){
         /*Do we have a t<->k<->h TP?  If so, add it to our count.*/
         L2th+=(IS_OUTEDGE(tail,k)&&IS_OUTEDGE(head,k)&&IS_OUTEDGE(k,head));
         if(htedge&&IS_OUTEDGE(head,k)&&IS_OUTEDGE(k,head)){ /*Only consider stats that could change*/
           L2kt=0;
           /*Now, count # u such that k<->u<->t (to get (k,t)'s ESP value)*/
-          for(f = EdgetreeMinimum(nwp->outedges, k); 
-          (u = nwp->outedges[f].value) != 0; f = EdgetreeSuccessor(nwp->outedges, f)) { 
+          STEP_THROUGH_OUTEDGES(k,f,u){ 
             if((u!=tail)&&(IS_OUTEDGE(u,k)))
               L2kt+=(IS_OUTEDGE(u,tail)&&IS_OUTEDGE(tail,u));  /*k<->u<->t?*/
           }
@@ -1005,14 +972,12 @@ void espRTP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
       }
     }
     /* step through outedges of tail (t->k: k!=h,h->t,k<->h)*/
-    for(e = EdgetreeMinimum(nwp->outedges, tail); 
-    (k = nwp->outedges[e].value) != 0; e = EdgetreeSuccessor(nwp->outedges, e)) {
+    STEP_THROUGH_OUTEDGES(tail,e,k) {
       if(k!=head){
         if(htedge&&IS_OUTEDGE(head,k)&&IS_OUTEDGE(k,head)){ /*Only consider stats that could change*/
           L2tk=0;
           /*Now, count # u such that k<->u<->t (to get (tk)'s ESP value)*/
-          for(f = EdgetreeMinimum(nwp->outedges, k); 
-          (u = nwp->outedges[f].value) != 0; f = EdgetreeSuccessor(nwp->outedges, f)) { 
+          STEP_THROUGH_OUTEDGES(k,f,u){ 
             if((u!=tail)&&(IS_OUTEDGE(u,k)))
               L2tk+=(IS_OUTEDGE(u,tail)&&IS_OUTEDGE(tail,u));  /*k<->u<->t?*/
           }
@@ -1025,14 +990,12 @@ void espRTP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
       }
     }
     /* step through inedges of head (k->h: k!=t,h->t,k<->t)*/
-    for(e = EdgetreeMinimum(nwp->inedges, head); 
-    (k = nwp->inedges[e].value) != 0; e = EdgetreeSuccessor(nwp->inedges, e)) {
+    STEP_THROUGH_INEDGES(head,e,k){
       if(k!=tail){
         if(htedge&&IS_OUTEDGE(tail,k)&&IS_OUTEDGE(k,tail)){ /*Only consider stats that could change*/
           L2kh=0;
           /*Now, count # u such that k<->u<->h (to get k->h's ESP value)*/
-          for(f = EdgetreeMinimum(nwp->outedges, k); 
-          (u = nwp->outedges[f].value) != 0; f = EdgetreeSuccessor(nwp->outedges, f)) { 
+          STEP_THROUGH_OUTEDGES(k,f,u){ 
             if((u!=head)&&(IS_OUTEDGE(u,k)))
               L2kh+=(IS_OUTEDGE(u,head)&&IS_OUTEDGE(head,u));  /*k<->u<->h?*/
           }
@@ -1045,14 +1008,12 @@ void espRTP_calc(Edge ntoggles, Vertex *tails, Vertex *heads, ModelTerm *mtp, Ne
       }
     }
     /* step through outedges of head (h->k: k!=t,h->t,k<->t)*/
-    for(e = EdgetreeMinimum(nwp->outedges, head); 
-    (k = nwp->outedges[e].value) != 0; e = EdgetreeSuccessor(nwp->outedges, e)) {
+    STEP_THROUGH_OUTEDGES(head,e,k){
       if(k!=tail){
         if(htedge&&IS_OUTEDGE(tail,k)&&IS_OUTEDGE(k,tail)){ /*Only consider stats that could change*/
           L2hk=0;
           /*Now, count # u such that k<->u<->h (to get h->k's ESP value)*/
-          for(f = EdgetreeMinimum(nwp->outedges, k); 
-          (u = nwp->outedges[f].value) != 0; f = EdgetreeSuccessor(nwp->outedges, f)) { 
+          STEP_THROUGH_OUTEDGES(k,f,u){ 
             if((u!=head)&&(IS_OUTEDGE(u,k)))
               L2hk+=(IS_OUTEDGE(u,head)&&IS_OUTEDGE(head,u));  /*k<->u<->h?*/
           }
@@ -1095,7 +1056,7 @@ Type codes are as follows (where (i,j) is the focal edge):
 Only one type may be specified per esp term.  UTP should always be used for undirected graphs; OTP is the traditional directed default.
 */
 D_CHANGESTAT_FN(d_desp) { 
-  int i,type;
+  int type;
   double *dvec,*cs;
   
   /*Set things up*/
@@ -1143,8 +1104,8 @@ D_CHANGESTAT_FN(d_dgwesp) {
   oneexpa = 1.0-exp(-alpha);    /*Precompute (1-exp(-alpha))*/
   type=(int)INPUT_PARAM[1];     /*Get the ESP type code to be used*/
   maxesp=(int)INPUT_PARAM[2];   /*Get the max ESP cutoff to use*/
-  cs=Calloc(maxesp,double);     /*Allocate memory for the ESP changescores*/
-  dvec=Calloc(maxesp,double);   /*Allocate memory for the ESP vals*/
+  cs=alloca(maxesp*sizeof(double));     /*Allocate memory for the ESP changescores*/
+  dvec=alloca(maxesp*sizeof(double));   /*Allocate memory for the ESP vals*/
   for(i=0;i<maxesp;i++)         /*Initialize the ESP vals*/
     dvec[i]=i+1.0;
 
@@ -1166,10 +1127,6 @@ D_CHANGESTAT_FN(d_dgwesp) {
     }
   }
   CHANGE_STAT[0]*=exp(alpha);
-  
-  /*Free the changescore memory*/
-  Free(cs);
-  Free(dvec);
 }
 
 
@@ -1201,8 +1158,8 @@ D_CHANGESTAT_FN(d_dnsp) {
   ZERO_ALL_CHANGESTATS(i);
   type=(int)INPUT_PARAM[0];     /*Get the ESP type code to be used*/
   dvec=INPUT_PARAM+1;           /*Get the pointer to the ESP stats list*/
-  cs_esp=Calloc(N_CHANGE_STATS,double);     /*Allocate memory for the DSP changescores*/
-  cs_dsp=Calloc(N_CHANGE_STATS,double);     /*Allocate memory for the DSP changescores*/
+  cs_esp=alloca(N_CHANGE_STATS*sizeof(double));     /*Allocate memory for the DSP changescores*/
+  cs_dsp=alloca(N_CHANGE_STATS*sizeof(double));     /*Allocate memory for the DSP changescores*/
 
   /*Obtain the ESP changescores (by type)*/
   switch(type){
@@ -1235,10 +1192,6 @@ D_CHANGESTAT_FN(d_dnsp) {
   
   for(i=0;i<N_CHANGE_STATS;i++)
     CHANGE_STAT[i]=(cs_dsp[i]-cs_esp[i]);
-    
-  /*Free the changescore memory*/
-  Free(cs_esp);
-  Free(cs_dsp);
 }
 
 
@@ -1268,12 +1221,12 @@ D_CHANGESTAT_FN(d_dgwnsp) {
   oneexpa = 1.0-exp(-alpha);    /*Precompute (1-exp(-alpha))*/
   type=(int)INPUT_PARAM[1];     /*Get the ESP type code to be used*/
   maxesp=(int)INPUT_PARAM[2];   /*Get the max ESP cutoff to use*/
-  cs_esp=Calloc(maxesp,double);     /*Allocate memory for the ESP changescores*/
-  dvec=Calloc(maxesp,double);   /*Allocate memory for the ESP vals*/
+  cs_esp=alloca(maxesp*sizeof(double));     /*Allocate memory for the ESP changescores*/
+  dvec=alloca(maxesp*sizeof(double));   /*Allocate memory for the ESP vals*/
   for(i=0;i<maxesp;i++)         /*Initialize the ESP vals*/
     dvec[i]=i+1.0;
   
-  cs_dsp=Calloc(maxesp,double);     /*Allocate memory for the ESP changescores*/
+  cs_dsp=alloca(maxesp*sizeof(double));     /*Allocate memory for the ESP changescores*/
 
   /*Obtain the changescores (by type)*/
   switch(type){
@@ -1309,10 +1262,6 @@ D_CHANGESTAT_FN(d_dgwnsp) {
       CHANGE_STAT[0]+=(1.0-pow(oneexpa,dvec[i]))*(cs_dsp[i]-cs_esp[i]);
   CHANGE_STAT[0]*=exp(alpha);
 
-  /*Free the changescore memory*/
-  Free(cs_esp);
-  Free(cs_dsp);
-  Free(dvec);
 }
 
 
